﻿Imports System.Text
Imports Newtonsoft.Json
Imports Newtonsoft.Json.Linq

Imports System.Net
Imports System.Web
Imports System.IO


Module Module1

    Dim endpoint As String = "https://www.bikereg.com/api/director/"

    Sub Main()
        Dim password As String = "mypass"
        Dim username As String = "myuser"

        Dim token As String = DoLogin(username, password)
        Dim events As List(Of Integer) = ListEvents(token)

        Dim categories As List(Of String) = GetCategories(token, events(0))

        GetEntries(token, events(0))
    End Sub

    Private Function GetEntries(token As String, eventID As Integer) As List(Of JToken)
        Dim dest As String = endpoint + "EventEntries"

        Dim request As HttpWebRequest = HttpWebRequest.CreateHttp(dest)
        request.Method = "POST"
        request.ContentType = "application/json"
        Dim params As New Dictionary(Of String, Object)

        params.Add("token", token)
        params.Add("eventID", eventID)

        'get all the entries in the last month
        params.Add("startDate", Now() - TimeSpan.FromDays(30))
        params.Add("endDate", Now())


        EncodePostParams(request, params)

        Dim entries As JArray = GetRespBody(Of JArray)(request, "GetEventEntriesResult")

        Dim regList As New List(Of JToken)

        For Each jo As JToken In entries
            regList.Add(jo)
        Next
        Return regList

    End Function

    Private Function GetCategories(token As String, eventID As Integer) As List(Of String)
        Dim dest As String = endpoint + "EventCategories"

        Dim request As HttpWebRequest = HttpWebRequest.CreateHttp(dest)
        request.Method = "POST"
        request.ContentType = "application/json"
        Dim params As New Dictionary(Of String, Object)
        params.Add("token", token)
        params.Add("eventID", eventID)

        EncodePostParams(request, params)

        Dim events As JArray = GetRespBody(Of JArray)(request, "GetEventCategoriesResult")

        Dim categories As New List(Of String)

        For Each jo As JObject In events
            categories.Add(jo.Value(Of String)("CategoryName"))
        Next
        Return categories
    End Function

    Private Function ListEvents(token As String) As List(Of Integer)

        Dim dest As String = endpoint + "PromoterEvents"

        Dim request As HttpWebRequest = HttpWebRequest.CreateHttp(dest)
        request.Method = "POST"
        request.ContentType = "application/json"
        Dim params As New Dictionary(Of String, Object)
        params.Add("token", token)

        EncodePostParams(request, params)

        Dim events As JArray = GetRespBody(Of JArray)(request, "GetPromoterEventsResult")

        Dim eventlist As New List(Of Integer)

        For Each jo As JObject In events
            eventlist.Add(jo.Value(Of Integer)("EventID"))
        Next

        Return eventlist



    End Function

    Private Function DoLogin(username As String, password As String) As String
        Dim dest As String = endpoint + "PromoterLogin"

        Dim request As HttpWebRequest = CreateRequest(dest)

        Dim params As New Dictionary(Of String, Object)
        params.Add("username", username)
        params.Add("password", password)

        EncodePostParams(request, params)

        Dim jtoken As JObject = GetRespBody(Of JObject)(request, "PromoterLoginResult")

        Return jtoken("authToken").Value(Of String)

    End Function
    Private Function CreateRequest(url) As HttpWebRequest

        Dim req As HttpWebRequest = HttpWebRequest.CreateHttp(url)
        req.Method = "POST"
        req.ContentType = "application/json"
        Return req

    End Function
    Private Sub EncodePostParams(ByVal webRequest As Net.WebRequest, ByVal params As Dictionary(Of String, Object))

        Dim microsoftDateFormatSettings As JsonSerializerSettings = New JsonSerializerSettings()
        With microsoftDateFormatSettings
            .DateFormatHandling = DateFormatHandling.MicrosoftDateFormat
        End With


        Dim json As String = JsonConvert.SerializeObject(params, microsoftDateFormatSettings)

        Using s As Stream = webRequest.GetRequestStream()
            s.Write(Encoding.UTF8.GetBytes(json), 0, json.Length)
        End Using
    End Sub
    Private Function GetRespBody(Of T)(req As HttpWebRequest, responseBodyName As String) As T
        Dim response As HttpWebResponse = req.GetResponse()
        Dim reader As New StreamReader(response.GetResponseStream())

        Dim json As String = reader.ReadToEnd

        'wrapped response body style forces us to deserialize twice, sadly
        Dim obj As JObject = JsonConvert.DeserializeObject(json)

        Return DirectCast(JsonConvert.DeserializeObject(obj.Value(Of String)(responseBodyName)), T)
    End Function

End Module
